import 'dart:math';
import 'package:nui_core/nui_core.dart';
import 'nui_math_opt.dart';

class NUIEqBrac{
  static const List<NUIMathOpt> OPERATOR_LEVEL = [
    NUIMathOpt.POWER,
    NUIMathOpt.REMAINDER,
    NUIMathOpt.MULTIPLY,
    NUIMathOpt.DIVIDE,
    NUIMathOpt.ADD,
    NUIMathOpt.MINUS
  ];

  NUIEqBrac.empty();
  NUIEqBrac({this.valueList, this.optList, this.parentCalBrac}){
    this.valueList = this.valueList ?? List();
    this.optList = this.optList ?? List();
  }

  List<dynamic> valueList;
  List<NUIMathOpt> optList;
  NUIEqBrac parentCalBrac;

  @override
  String toString(){
    return "($optList, $valueList)";
  }

  double calculate(){
    if(isNullOrEmpty(valueList)){
      throwNUI("NUIFlexLogic", "NUIEqBrac -> value list should not be empty for NUIEqBrac");
    }

    for(NUIMathOpt opt in OPERATOR_LEVEL){
      _optByLevel(opt);
    }

    var processedEq = "";
    dynamic firstItem = valueList[0];
    valueList.removeAt(0);

    double currentVal = firstItem is NUIEqBrac ? firstItem.calculate() : firstItem as double;
    processedEq += currentVal.toString();

    var index = 0;
    for(dynamic v in valueList){
      NUIMathOpt opt = optList[index];

      double nextVal;
      if(v is NUIEqBrac){
        nextVal = v.calculate();
      }
      else{
        nextVal = v;
      }

      processedEq += opt.value();
      processedEq += nextVal.toString();

      currentVal = _opt(opt, currentVal, nextVal);
      index++;
    }

    return currentVal;
  }

  void _optByLevel(NUIMathOpt operator){
    List<dynamic> cloneValueList = List();
    cloneValueList.addAll(valueList);
    List<NUIMathOpt> cloneOptList = List();
    cloneOptList.addAll(optList);

    bool finishAll = true;

    int index = 0;
    for(NUIMathOpt op in optList){
      if(op == operator){
        //Handle this operator first

        double firstVal = _getValue(cloneValueList[index]);
        double calcValue;
        bool isSingleValue = false;
        if(cloneValueList.length > index + 1){
          double secondVal = _getValue(cloneValueList[index + 1]);
          calcValue = _opt(op, firstVal, secondVal);
        }
        else{
          isSingleValue = true;
          if(op == NUIMathOpt.MINUS){
            //probably negative values
            calcValue = _opt(op, 0.0, firstVal);
          }
          else{
            throwNUI("NUIFlexLogic", "NUIEqBrac -> Equation is not complete");
          }
        }

        cloneValueList.removeAt(index);
        if(!isSingleValue) {
          cloneValueList.removeAt(index);
        }

        cloneValueList.insert(index, calcValue);
        cloneOptList.remove(index);

        this.valueList.clear();
        this.valueList.addAll(cloneValueList);

        this.optList.clear();
        this.optList.addAll(cloneOptList);

        finishAll = false;
        break;
      }
      index++;
    }

    if(!finishAll && (operator == NUIMathOpt.MINUS && valueList.length > 1)){
      _optByLevel(operator);
    }

  }

  double _getValue(Object value){
    if(value is NUIEqBrac){
      return value.calculate();
    }
    return value;
  }

  double _opt(NUIMathOpt opt, double curVal, double nextVal){
    if(opt == NUIMathOpt.ADD) {
      return curVal + nextVal;
    }
    else if(opt == NUIMathOpt.MINUS) {
      return curVal - nextVal;
    }
    else if(opt == NUIMathOpt.MULTIPLY) {
      return curVal * nextVal;
    }
    else if(opt == NUIMathOpt.DIVIDE) {
      return curVal / nextVal;
    }
    else if(opt == NUIMathOpt.POWER) {
      return pow(curVal, nextVal);
    }
    else if(opt == NUIMathOpt.REMAINDER) {
      return curVal%nextVal;
    }
    else {
      throwNUI("NUIFlexLogic", "NUIEqBrac -> Invalid operator of $opt");
    }
  }
}

