
import 'package:nui_core/code/nui_code_util.dart';
import 'package:nui_core/data/nui_data_util.dart';
import 'package:nui_config_logic/condition/nui_condition_value.dart';
import 'package:nui_config_logic/nui_config_logic.dart';

class NUIEqDecoder {

  static double decode(String equation, Map<String, NUIConditionValue> map, NUIConfigLogic mapper) {
    logNUI("NUIEqDecoder", "Decoding equation : $equation");
    String curEquation = equation;

    NUIEqBrac baseCalBrac = NUIEqBrac();
    NUIEqBrac curCalBrac = baseCalBrac;
    String curValue = "";
    double curDouble;

    curEquation = curEquation.replaceAll(" ", "");

    String processedEq = "";

    int index = 0;
    for (int i = 0; i<curEquation.length; i++) {
      final c = curEquation[i];
      String charac = c;

      switch (charac) {
        case "(": {
          //Creating new bracket item
          NUIEqBrac newCalBrac = NUIEqBrac();
          curCalBrac.valueList.add(newCalBrac);
          newCalBrac.parentCalBrac = curCalBrac;
          curCalBrac = newCalBrac;
          curValue = "";
          curDouble = null;

          processedEq += "(";
          break;
        }
        case ")": {
          //Closing current bracket
          if (!isNullOrEmpty(curValue)) {
            double value = double.parse(curValue);
            curCalBrac.valueList.add(value);

            processedEq += value.toString();
          } else if (curDouble != null) {
            curCalBrac.valueList.add(curDouble);
          }
          curValue = "";
          curDouble = null;
          curCalBrac = curCalBrac.parentCalBrac;

          processedEq += ")";
          break;
        }
        case "<": {
          //Found starting char for variable key
          curValue = "";
          curDouble = null;
          break;
        }
        case ">": {
          //Get the value for the variable key
          double value = map[curValue].getDouble(null, null);
          curDouble = value;
          curValue = "";

          processedEq += curDouble.toString();
          break;
        }
        case "[": {
          //Found starting char for inner matrix calculation
          curValue = "";
          curDouble = null;
          break;
        }
        case "]": {
          //Get the value for the matrix key
          double value = mapper.getPrimitiveMatrixValue(curValue, double);
          curDouble = value;
          curValue = "";

          processedEq += curDouble.toString();
          break;
        }
        case NUIMathOperators.ADD: {
          curCalBrac.optList.add(NUIMathOpt.ADD);
          if (!isNullOrEmpty(curValue)) {
            double value = double.parse(curValue);
            curCalBrac.valueList.add(value);

            processedEq += value.toString();
          } else if (curDouble != null) {
            curCalBrac.valueList.add(curDouble);
          }
          curValue = "";
          curDouble = null;

          processedEq += "+";
          break;
        }
        case NUIMathOperators.MINUS: {
          curCalBrac.optList.add(NUIMathOpt.MINUS);
          if (!isNullOrEmpty(curValue)) {
            double value = double.parse(curValue);
            curCalBrac.valueList.add(value);

            processedEq += value.toString();
          } else if (curDouble != null) {
            curCalBrac.valueList.add(curDouble);
          }
          curValue = "";
          curDouble = null;

          processedEq += "-";
          break;
        }
        case NUIMathOperators.MULTIPLY: {
          curCalBrac.optList.add(NUIMathOpt.MULTIPLY);
          if (!isNullOrEmpty(curValue)) {
            double value = double.parse(curValue);
            curCalBrac.valueList.add(value);

            processedEq += value.toString();
          } else if (curDouble != null) {
            curCalBrac.valueList.add(curDouble);
          }
          curValue = "";
          curDouble = null;

          processedEq += "*";
          break;
        }
        case NUIMathOperators.DIVIDE: {
          curCalBrac.optList.add(NUIMathOpt.DIVIDE);
          if (!isNullOrEmpty(curValue)) {
            double value = double.parse(curValue);
            curCalBrac.valueList.add(value);

            processedEq += value.toString();
          } else if (curDouble != null) {
            curCalBrac.valueList.add(curDouble);
          }
          curValue = "";
          curDouble = null;

          processedEq += "/";
          break;
        }
        case NUIMathOperators.POWER: {
          curCalBrac.optList.add(NUIMathOpt.POWER);
          if (!isNullOrEmpty(curValue)) {
            double value = double.parse(curValue);
            curCalBrac.valueList.add(value);

            processedEq += value.toString();
          } else if (curDouble != null) {
            curCalBrac.valueList.add(curDouble);
          }
          curValue = "";
          curDouble = null;

          processedEq += "^";
          break;
        }
        case NUIMathOperators.REMAINDER: {
          curCalBrac.optList.add(NUIMathOpt.REMAINDER);
          if (!isNullOrEmpty(curValue)) {
            double value = double.parse(curValue);
            curCalBrac.valueList.add(value);

            processedEq += value.toString();
          } else if (curDouble != null) {
            curCalBrac.valueList.add(curDouble);
          }
          curValue = "";
          curDouble = null;

          processedEq += "%";
          break;
        }
        default: {
          //Forging number string
          curValue += charac;
          break;
        }
      }
    }

    if (!isNullOrEmpty(curValue)) {
      double value = double.parse(curValue);
      curCalBrac.valueList.add(value);
      processedEq += value.toString();
    } else if (curDouble != null) {
      curCalBrac.valueList.add(curDouble);
    }

    logNUI("NUIEqDecoder", "Processed EQ : $processedEq, value list : ${baseCalBrac.valueList.length}, opt list : ${baseCalBrac.optList.length}");

    logNUI("NUIEqDecoder", "Finalized equation : $baseCalBrac");

    return baseCalBrac.calculate();
  }
}