import 'package:nui_core/code/nui_code_util.dart';
import 'package:nui_core/data/nui_ent.dart';
import 'package:nui_core/string/nui_string_util.dart';
import 'package:nui_config_logic/nui_config_logic.dart';

import 'nui_condition_compare.dart';
import 'nui_condition_value.dart';
import 'nui_conditional_operator.dart';

class NUIConditionCombination extends NUIEnt<NUIConditionCombination>{
  NUIConditionCombination.empty();

  String result;
  NUIConditionValue resultRaw ;
  List<String> resultValues = List();
  List<NUIConditionValue> resultValuesRaw = List();
  List<NUIConditionCompare> comparisons = List();
  String logic = NUIConditionalOpt.AND.value();
  NUIConditionCombination subComparison;

  bool compare(Map<String, NUIConditionValue> valueMap, NUIConfigLogic mapper){
    resultRaw = NUIConditionValue.fromRawSingle(raw: result, value: resultRaw);

    Map<String, NUIConditionValue> map = valueMap != null ? valueMap : Map();
    if(match(logic, NUIConditionalOpt.AND.value())) {
      bool result = true;
      for(NUIConditionCompare com in (comparisons != null ? comparisons : List<NUIConditionCompare>())){
        result = result && com.compareInternal(map, mapper);
      }
      if(subComparison != null){
        result = result && subComparison.compare(map, mapper);
      }
      return result;
    }
    else if(match(logic, NUIConditionalOpt.OR.value())) {
      bool result = false;
      for(NUIConditionCompare com in (comparisons != null ? comparisons : List<NUIConditionCompare>())){
        result = result || com.compareInternal(map, mapper);
      }
      if(subComparison != null){
        result = result || subComparison.compare(map, mapper);
      }
      return result;
    }
    else {
      throwNUI("NUIConfigLogic", "IConditionCombination -> Invalid condition operator found : " + logic);
    }
  }

  NUIConditionValue getResultRaw() {
    resultRaw = NUIConditionValue.fromRawSingle(raw: result, value: resultRaw);
    resultRaw.setValuesRaw(NUIConditionValue.fromRawList(valuesRaw: resultValues, values: resultValuesRaw, isValueKey: true));
    return resultRaw;
  }

  void setResultRaw(NUIConditionValue resultRaw) {
    this.resultRaw = resultRaw;
  }

  @override
  NUIEntMapper<NUIConditionCombination> mapper() => NUIConditionCombinationEnt();
}


class NUIConditionCombinationEnt extends NUIEntMapper<NUIConditionCombination>{
  @override
  String entityName() => "NUIConditionCombination";

  @override
  List<NUIEntField> fields() {
    return [
      NUIEntField<NUIConditionCombination>(name: "result", type: NUIEntType.STRING, setter: (data, value) => data.result = value, getter: (data) => data.result),
      NUIEntField<NUIConditionCombination>(name: "logic", type: NUIEntType.STRING, setter: (data, value) => data.logic = value, getter: (data) => data.logic),
      NUIEntField<NUIConditionCombination>(name: "resultRaw", type: NUIEntType.OBJECT, setter: (data, value) => data.resultRaw = asObject(NUIConditionValueEnt(), value), getter: (data) => data.resultRaw),
      NUIEntField<NUIConditionCombination>(name: "resultValues", type: NUIEntType.LIST_STRING, setter: (data, value) => data.resultValues = asList<String>(value), getter: (data) => data.resultValues),
      NUIEntField<NUIConditionCombination>(name: "resultValuesRaw", type: NUIEntType.LIST_OBJECT, setter: (data, value) => data.resultValuesRaw = asListOfObjects(NUIConditionValueEnt(), value), getter: (data) => data.resultValuesRaw),
      NUIEntField<NUIConditionCombination>(name: "comparisons", type: NUIEntType.LIST_OBJECT, setter: (data, value) => data.comparisons = asListOfObjects(NUIConditionCompareEnt(), value), getter: (data) => data.comparisons),
      NUIEntField<NUIConditionCombination>(name: "subComparison", type: NUIEntType.OBJECT, setter: (data, value) => data.subComparison = asObject(NUIConditionCombinationEnt(), value), getter: (data) => data.subComparison),
    ];
  }

  @override
  NUIConditionCombination newInstance() => NUIConditionCombination.empty();
}