import 'dart:core';
import 'package:nui_core/code/nui_code_util.dart';
import 'package:nui_core/data/nui_data_util.dart';
import 'package:nui_core/datetime/nui_dt_util.dart';
import 'package:nui_core/nui_core.dart';
import 'package:nui_config_logic/calculation/nui_eq_decoder.dart';
import 'package:nui_config_logic/data/nui_fl_data_type.dart';
import 'package:nui_config_logic/data/nui_value_raw.dart';
import 'package:nui_config_logic/nui_config_logic.dart';

class NUIConditionValue extends NUIEnt<NUIConditionValue> {

  static NUIConditionValue fromRawSingle({String raw, NUIConditionValue value}) {
    if (isNullOrEmpty(raw)) return value;
    return NUIValueRaw(rawValue: raw);
  }

  static List<NUIConditionValue> fromRawList({List<String> valuesRaw, List<NUIConditionValue> values, bool isValueKey: false}) {
    if (isNullOrEmpty(valuesRaw))
      return values;
    else {
      List<NUIConditionValue> newValues = new List();
      for (String s in valuesRaw) {
        newValues.add(NUIValueRaw(rawValue: s, isKeyMap: isValueKey));
      }
      return newValues;
    }
  }

  NUIConditionValue.empty();

  NUIConditionValue({String dataType, Object value, this.isKey, this.id}) {
    this.value = value.toString();
    this.dataType = dataType ?? NUILogicDataType.STRING.value();
  }

  String dataType = NUILogicDataType.STRING.value(); //Compulsory
  String value; //Compulsory
  List<NUIConditionValue> _valuesRaw; //For list of results
  List<String> values;
  String id;
  bool isKey;
  String valueKey;
  String calculationEq;

  NUIConditionValue getValueFromMap(Map<String, NUIConditionValue> map) {
    if (!map.containsKey(value)) {
      throwNUI("NUIConfigLogic", "NUIConditionValue -> Value for key : $value is not found in the value map list");
    }
    return map[value];
  }

  String getString(Map<String, NUIConditionValue> valueMap, NUIConfigLogic mapper) {
    Map<String, NUIConditionValue> map = valueMap != null ? valueMap : Map<String, NUIConditionValue>();
    if (!(isKey ?? false)) {
      return value != null ? value : "";
    }
    else {
      NUIConditionValue value = getValueFromMap(map);
      return value.getString(null, null);
    }
  }

  int getInt(Map<String, NUIConditionValue> valueMap, NUIConfigLogic mapper) {
    _valuesRaw = NUIConditionValue.fromRawList(valuesRaw: values, values: _valuesRaw);
    Map<String, NUIConditionValue> map = valueMap != null ? valueMap : Map();
    if (!(isKey ?? false)) {
      if (calculationEq == null || isNullOrEmpty(calculationEq)) {
        if (_valuesRaw == null) {
          return int.parse(value);
        }
        else {
          return calculateInnerValues(map).toInt();
        }
      }
      else {
        return NUIEqDecoder.decode(calculationEq, map, mapper).toInt();
      }
    }
    else {
      NUIConditionValue value = getValueFromMap(map);
      return value.getInt(map, mapper);
    }
  }

  double getDouble(Map<String, NUIConditionValue> valueMap, NUIConfigLogic mapper) {
    _valuesRaw = NUIConditionValue.fromRawList(valuesRaw: values, values: _valuesRaw);
    Map<String, NUIConditionValue> map = valueMap != null ? valueMap : new Map();
    if (!(isKey ?? false)) {
      if (calculationEq == null || isNullOrEmpty(calculationEq)) {
        if (isNullOrEmpty(_valuesRaw)) {
          return double.parse(value);
        }
        else {
          return calculateInnerValues(map);
        }
      }
      else {
        return NUIEqDecoder.decode(calculationEq, map, mapper);
      }
    }
    else {
      NUIConditionValue value = getValueFromMap(map);
      return value.getDouble(map, mapper);
    }
  }

  int getLong(Map<String, NUIConditionValue> valueMap, NUIConfigLogic mapper) {
    _valuesRaw = NUIConditionValue.fromRawList(valuesRaw: values, values: _valuesRaw);
    Map<String, NUIConditionValue> map = valueMap != null ? valueMap : Map();
    if (!(isKey ?? false)) {
      if (isNullOrEmpty(calculationEq)) {
        if (_valuesRaw == null) {
          return int.parse(value);
        }
        else {
          return calculateInnerValues(map).toInt();
        }
      }
      else {
        return NUIEqDecoder.decode(calculationEq, map, mapper).toInt();
      }
    }
    else {
      NUIConditionValue value = getValueFromMap(map);
      return value.getLong(map, mapper);
    }
  }

  DateTime getDate(Map<String, NUIConditionValue> valueMap, NUIConfigLogic mapper) {
    Map<String, NUIConditionValue> map = valueMap != null ? valueMap : Map();
    if (!(isKey ?? false)) {
      return stringToDate(value);
    }
    else {
      NUIConditionValue value = getValueFromMap(map);
      return value.getDate(null, null);
    }
  }

  bool getBool(Map<String, NUIConditionValue> valueMap, NUIConfigLogic mapper) {
    Map<String, NUIConditionValue> map = valueMap != null ? valueMap : Map();
    if (!(isKey ?? false)) {
      return match(value, "true");
    }
    else {
      NUIConditionValue value = getValueFromMap(map);
      return value.getBool(null, null);
    }
  }

  bool getIsNull(Map<String, NUIConditionValue> valueMap, NUIConfigLogic mapper) {
    Map<String, NUIConditionValue> map = valueMap != null ? valueMap : Map();
    if (!(isKey ?? false)) {
      return false;
    }
    else {
      return !map.containsKey(value);
    }
  }

  bool getIsNullOrEmpty(Map<String, NUIConditionValue> map, NUIConfigLogic mapper) {
    if (!(isKey ?? false)) {
      return false;
    }
    else {
      return !map.containsKey(value) || (map[value]) != null && isNullOrEmpty(map[value].value);
    }
  }

  double calculateInnerValues(Map<String, NUIConditionValue> map) {
    _valuesRaw = NUIConditionValue.fromRawList(valuesRaw: values, values: _valuesRaw);
    List<NUIConditionValue> toCalValues = List()..addAll(_valuesRaw);
    if (!isNullOrEmpty(_valuesRaw)) {
      double initialValue = toCalValues[0].getDouble(map, null);
      toCalValues.removeAt(0);

      for (NUIConditionValue v in toCalValues) {
        initialValue += v.getDouble(map, null);
      }

      return initialValue;
    }
    return 0.0;
  }

  List<NUIConditionValue> getValuesRaw() {
    if (isNullOrEmpty(_valuesRaw) && !isNullOrEmpty(values)) {
      List<NUIConditionValue> newValues = List();
      for (String s in values) {
        newValues.add(NUIValueRaw(rawValue: s));
      }
      return newValues;
    }
    else {
      return _valuesRaw;
    }
  }

  void setValuesRaw(List<NUIConditionValue> valuesRaw) {
    this._valuesRaw = valuesRaw;
  }

  NUIConditionValue getValueFromListByKey(String key) {
    if (isNullOrEmpty(_valuesRaw)) return null;

    for (NUIConditionValue c in _valuesRaw) {
      if (match(c.valueKey, key)) {
        return c;
      }
    }
    return null;
  }

  @override
  NUIEntMapper<NUIConditionValue> mapper() => NUIConditionValueEnt();
}

class NUIConditionValueEnt extends NUIEntMapper<NUIConditionValue>{
  @override
  String entityName() => "NUIConditionValue";

  @override
  List<NUIEntField> fields() {
    return [
      NUIEntField<NUIConditionValue>(name: "dataType", type: NUIEntType.STRING, setter: (data, value) => data.dataType = value, getter: (data) => data.dataType),
      NUIEntField<NUIConditionValue>(name: "value", type: NUIEntType.STRING, setter: (data, value) => data.value = value, getter: (data) => data.value),
      NUIEntField<NUIConditionValue>(name: "id", type: NUIEntType.STRING, setter: (data, value) => data.id = value, getter: (data) => data.id),
      NUIEntField<NUIConditionValue>(name: "valueKey", type: NUIEntType.STRING, setter: (data, value) => data.valueKey = value, getter: (data) => data.valueKey),
      NUIEntField<NUIConditionValue>(name: "calculationEq", type: NUIEntType.STRING, setter: (data, value) => data.calculationEq = value, getter: (data) => data.calculationEq),
      NUIEntField<NUIConditionValue>(name: "isKey", type: NUIEntType.BOOLEAN, setter: (data, value) => data.isKey = value, getter: (data) => data.isKey),
      NUIEntField<NUIConditionValue>(name: "valuesRaw", type: NUIEntType.LIST_OBJECT, setter: (data, value) => data._valuesRaw = asListOfObjects(NUIConditionValueEnt(), value), getter: (data) => data._valuesRaw),
      NUIEntField<NUIConditionValue>(name: "values", type: NUIEntType.LIST_STRING, setter: (data, value) => data.values = asList<String>(value), getter: (data) => data.values),
    ];
  }

  @override
  NUIConditionValue newInstance() => NUIConditionValue.empty();
}