import 'dart:core';
import 'package:nui_core/code/nui_code_util.dart';
import 'package:nui_core/data/nui_data_util.dart';
import 'package:nui_config_logic/condition/nui_condition_value.dart';
import 'package:nui_config_logic/data/nui_fl_data_type.dart';

class NUIValueRaw extends NUIConditionValue {

  String rawValue;
  bool isKeyMap;

  NUIValueRaw({this.rawValue, this.isKeyMap: false}){
    processRawValue();
  }

  void processRawValue(){
    if(isNullOrEmpty(rawValue)) throwNUI("NUIFlexLogic", "IValueRaw incorrect format");

    if(!isKeyMap) {
      this.isKey = (rawValue.startsWith("#"));

      if (rawValue.startsWith("~")) {
        //Is calculation matrix
        this.calculationEq = (rawValue.replaceFirst("~", ""));
        this.isKey = false;
        this.dataType = NUILogicDataType.DOUBLE.value();
        return;
      }

      int index = 0;
      String curValue = "";
      for (int i = 0; i<rawValue.length; i++) {
        final c = rawValue[i];
        switch (c) {
            case '[': {
            //Found starting char for inner matrix calculation
            curValue = "";
            break;
          }
          case ']': {
            //Get the value for the matrix key
            String innValue = curValue;

            if (index == 0) {
              //First parameter, set it as the value
              this.value = (innValue);
            } else if (index == 1) {
              //Second parameter, set it as the data type
              this.dataType = (innValue);
            }
            curValue = "";
            index++;
            break;
          }
          default: {
            //Forging number string
            curValue = curValue + c;
            break;
          }
        }
      }

      if (isNullOrEmpty(this.value)) throwNUI("NUIFlexLogic", "IValueRaw incorrect format - missing value");

      if (isNullOrEmpty(this.dataType)) {
        this.dataType = (NUILogicDataType.STRING.value());
      }
    }
    else{
      int index = 0;
      String curValue = "";

      for (int i = 0; i<rawValue.length; i++) {
        final c = rawValue[i];
        switch (c) {
          case '[': {
            //Found starting char for inner matrix calculation
            curValue = "";
            break;
          }
          case ']': {
            //Get the value for the matrix key
            String innValue = curValue;

            if(index == 0){
              //First parameter, set it as the key
              this.valueKey = (innValue);
            }
            if (index == 1) {
              //First parameter, set it as the value
              this.value = (innValue);
            } else if (index == 2) {
              //Second parameter, set it as the data type
              this.dataType = (innValue);
            }
            curValue = "";
            index++;
            break;
          }
          default: {
            //Forging number string
            curValue = curValue + c;
            break;
          }
        }
      }

      if(isNullOrEmpty(this.valueKey))
        throwNUI("NUIFlexLogic", "IValueRaw incorrect format - missing value key");

      if (isNullOrEmpty(this.dataType))
        throwNUI("NUIFlexLogic", "IValueRaw incorrect format - missing value");

      if (isNullOrEmpty(this.dataType)) {
        this.dataType = NUILogicDataType.STRING.value();
      }
    }
  }
}
