library nui_config_logic;

import 'package:nui_core/data/nui_data_util.dart';
import 'package:nui_core/data/nui_ent.dart';
import 'package:nui_config_logic/data/nui_fl_data_type.dart';
import 'package:nui_config_logic/data/nui_map_value.dart';
import 'package:nui_config_logic/data/nui_value_map_list.dart';
import 'condition/nui_condition_matrix.dart';
import 'condition/nui_condition_value.dart';
import 'data/nui_condition_keymap_list.dart';
import 'data/nui_value_raw.dart';

export 'package:nui_config_logic/calculation/nui_eq_brac.dart';
export 'package:nui_config_logic/calculation/nui_math_opt.dart';
export 'package:nui_config_logic/calculation/nui_eq_decoder.dart';

export 'package:nui_config_logic/condition/nui_comparison_operator.dart';
export 'package:nui_config_logic/condition/nui_condition_combination.dart';
export 'package:nui_config_logic/condition/nui_condition_compare.dart';
export 'package:nui_config_logic/condition/nui_condition_key_map.dart';
export 'package:nui_config_logic/condition/nui_condition_matrix.dart';
export 'package:nui_config_logic/condition/nui_condition_value.dart';
export 'package:nui_config_logic/condition/nui_conditional_operator.dart';

export 'package:nui_config_logic/data/nui_condition_keymap_list.dart';
export 'package:nui_config_logic/data/nui_fl_data_type.dart';
export 'package:nui_config_logic/data/nui_map_value.dart';
export 'package:nui_config_logic/data/nui_value_map_list.dart';
export 'package:nui_config_logic/data/nui_value_raw.dart';

class NUIConfigLogic {
  NUIConfigLogic._builder(NUIConfigLogicBuilder builder) {
    this.valueMapList = builder._valueMapList;
    this.conditionKeyMapList = builder._conditionKeyMapList;
  }

  NUIValueMapList valueMapList;
  NUIConditionKeyMapList conditionKeyMapList;

  void setValueListToMap(NUIConditionValue conditionValue){
    if(conditionValue != null && !isNullOrEmpty(conditionValue.getValuesRaw())){
      for(NUIConditionValue v in conditionValue.getValuesRaw()){
        if(!isNullOrEmpty(v.valueKey)){
          insertValue(v.valueKey, v);
        }
      }
    }
  }

  dynamic getPrimitiveMatrixValue(String key, Type dataType) {
    Map valueMap = this.valueMapList.listToMap();
    NUIConditionMatrix conMap = this.conditionKeyMapList.getConditionMapping(key);
    NUIConditionValue value = conMap != null ? conMap.getConditionResult(valueMap, this) : null;
    setValueListToMap(value);
    if (value != null) {
      if (dataType == String) {
        return value.getString(valueMap, this);
      }

      if (dataType == double) {
        return value.getDouble(valueMap, this);
      }

      if (dataType == int) {
        return value.getInt(valueMap, this);
      }

      if (dataType == bool) {
        return value.getBool(valueMap, this);
      }

      if (dataType == DateTime) {
        return value.getDate(valueMap, this);
      }
    }

    return null;
  }

  NUIConditionValue getMatrixValue(String key) {
    Map valueMap = this.valueMapList.listToMap();
    NUIConditionMatrix conMap = this.conditionKeyMapList.getConditionMapping(key);
    NUIConditionValue result = conMap != null ? conMap.getConditionResult(valueMap, this) : null;
    setValueListToMap(result);
    return result;
  }

  void insertValue(String key, NUIConditionValue value, {String id}) {
    List list = this.valueMapList.values;
    if (list != null) {
      list.add(NUIMapValue(key: key, valueRaw: value, id: id));
    }
  }

  NUIConditionValue getResultValueFromMap(String key){
    if(valueMapList == null) return null;

    List list = this.valueMapList.values;
    if (list != null) {
      return valueMapList.listToMap()[key];
    }
    return null;
  }

  dynamic getPrimitiveResultValueFromMap(String key, Type dataType) {
    if(valueMapList == null) return null;

    Map valueMap = this.valueMapList.listToMap();
    NUIConditionValue value = valueMap[key];
    setValueListToMap(value);
    if (value != null) {
      if (dataType == String) {
        return value.getString(valueMap, this);
      }

      if (dataType == double) {
        return value.getDouble(valueMap, this);
      }

      if (dataType == int) {
        return value.getInt(valueMap, this);
      }

      if (dataType == bool) {
        return value.getBool(valueMap, this);
      }

      if (dataType == DateTime) {
        return value.getDate(valueMap, this);
      }
    }

    return null;
  }

  void insertValueCommand(String key, String value){
    insertValue(key, NUIValueRaw(rawValue: value));
  }

  void insertValueListCommand(String key, List<String> values){
    if(isNullOrEmpty(values)) return;

    NUIConditionValue value = NUIConditionValue();
    NUIConditionValue firstValue = NUIValueRaw(rawValue: values[0]);
    value.dataType = (firstValue.dataType);

    List<String> valueList = List();
    for(String s in values){
      valueList.add(s);
    }

    value.values = (valueList);
    insertValue(key, value);
  }

  void insertValueList(String key, List<NUIConditionValue> values){
    if(isNullOrEmpty(values)) return;

    NUIConditionValue value = NUIConditionValue();
    NUIConditionValue firstValue = values[0];
    value.dataType = (firstValue.dataType);
    value.setValuesRaw(values);
    insertValue(key, value);
  }

  void insertValues(Map<String, dynamic> jsonObject) {
    for(String key in jsonObject.keys){
      dynamic value = jsonObject[key];
      insertDynamicValue(key, value, null, null);
    }
  }

  void insertDynamicValue(String key, Object value, String id, List<dynamic> conditionValues) {
    if (value is String) {
      if (conditionValues == null) {
        this.insertValue(key, NUIConditionValue(dataType: NUILogicDataType.STRING.value(), value: value, isKey: false, id: id));
      } else {
        conditionValues.add(NUIConditionValue(dataType: NUILogicDataType.STRING.value(), value: value, isKey: false));
      }
    } else if (value is double) {
      if (conditionValues == null) {
        this.insertValue(key, NUIConditionValue(dataType: NUILogicDataType.DOUBLE.value(), value: value, isKey: false, id: id));
      } else {
        conditionValues.add(NUIConditionValue(dataType: NUILogicDataType.DOUBLE.value(), value: value, isKey: false));
      }
    } else if (value is int) {
      if (conditionValues == null) {
        this.insertValue(key, NUIConditionValue(dataType : NUILogicDataType.INT.value(), value: value, isKey: false, id: id));
      } else {
        conditionValues.add(NUIConditionValue(dataType: NUILogicDataType.INT.value(), value: value, isKey: false));
      }
    } else if (value is bool) {
      if (conditionValues == null) {
        this.insertValue(key, NUIConditionValue(dataType: NUILogicDataType.BOOL.value(), value: value, isKey: false, id: id));
      } else {
        conditionValues.add(NUIConditionValue(dataType: NUILogicDataType.BOOL.value(), value: value, isKey: false));
      }
    } else if (value is DateTime) {
      if (conditionValues == null) {
        this.insertValue(key, NUIConditionValue(dataType: NUILogicDataType.DATE.value(), value: value, isKey: false, id: id));
      } else {
        conditionValues.add(NUIConditionValue(dataType: NUILogicDataType.DATE.value(), value: value, isKey: false));
      }
    } else {
      NUIConditionValue conditionValue;
      List subValues;
      if (value is List) {
        conditionValue = NUIConditionValue(dataType: NUILogicDataType.LIST.value(), value: "", isKey: false);
        subValues = List();

        for(dynamic innerValue in value){
          this.insertDynamicValue(key, innerValue, id, subValues);
        }

        conditionValue.setValuesRaw(subValues);
        this.insertValue(key, conditionValue, id: id);
      }
    }

  }

  void insertConditionValues(String key, List<NUIConditionValue> values) {
    NUIConditionValue parentValue = NUIConditionValue(dataType: NUILogicDataType.LIST.value(), value: "", isKey: false);
    List subValues = List();

    for(NUIConditionValue v in values){
      subValues.add(v);
    }

    parentValue.setValuesRaw(subValues);
    insertValue(key, parentValue);
  }

}

class NUIConfigLogicBuilder {
  NUIValueMapList _valueMapList = NUIValueMapList.empty();
  NUIConditionKeyMapList _conditionKeyMapList = NUIConditionKeyMapList.empty();

  NUIConfigLogicBuilder valueMapList(NUIValueMapList mapList) {
    this._valueMapList = mapList;
    return this;
  }

  NUIConfigLogicBuilder valueMapListFromJson(String json) {
    this._valueMapList = asObject(NUIValueMapListEnt(), json);
    return this;
  }

  NUIConfigLogicBuilder conditionKeyMapList(NUIConditionKeyMapList mapList) {
    this._conditionKeyMapList = mapList;
    return this;
  }

  NUIConfigLogicBuilder conditionKeyMapListFromJson(String json) {
    this._conditionKeyMapList = asObject(NUIConditionKeyMapListEnt(), json);
    return this;
  }

  NUIConfigLogic build() {
    return NUIConfigLogic._builder(this);
  }
}
